import copy
import decimal

from django.apps import apps
from django.db.models import ObjectDoesNotExist
from django.template.exceptions import TemplateDoesNotExist
from django.template.loader import select_template
from django.utils.functional import lazy
from django.utils.safestring import mark_safe

from . import settings

global_options = settings.FORM_OPTIONS


def get_option(form, option, default=None):
    form_options = getattr(getattr(form, "Meta", None), "options", {})
    return form_options.get(option, global_options.get(option, default))


def get_related_object(scope, field_name):
    """
    Returns the related field, referenced by the content of a ModelChoiceField.
    """
    try:
        Model = apps.get_model(scope[field_name]["model"])
        relobj = Model.objects.get(pk=scope[field_name]["pk"])
    except (ObjectDoesNotExist, LookupError):
        relobj = None
    return relobj


def insert_fields(
    fieldsets, new_fields, block=None, position=-1, blockname=None, blockattrs=None
):
    """
    creates a copy of fieldsets inserting the new fields either in the indexed block at the position,
    or - if no block is given - at the end
    """
    if blockattrs is None:
        blockattrs = dict()
    if block is None:
        fs = (
            list(fieldsets[:position] if position != -1 else fieldsets)
            + [
                (
                    blockname,
                    {
                        "classes": ("collapse",) if len(fieldsets) > 0 else (),
                        "fields": list(new_fields),
                        **blockattrs,
                    },
                )
            ]
            + list(fieldsets[position:] if position != -1 else [])
        )
        return fs
    modify = copy.deepcopy(fieldsets[block])
    fields = modify[1]["fields"]
    if position >= 0:
        modify[1]["fields"] = (
            list(fields[:position]) + list(new_fields) + list(fields[position:])
        )
    else:
        modify[1]["fields"] = (
            list(fields[: position + 1] if position != -1 else fields)
            + list(new_fields)
            + list(fields[position + 1 :] if position != -1 else [])
        )
    fs = (
        list(fieldsets[:block] if block != -1 else fieldsets)
        + [modify]
        + list(fieldsets[block + 1 :] if block != -1 else [])
    )
    return fs


def first_choice(choices):
    for value, verbose in choices:
        if not isinstance(verbose, (tuple, list)):
            return value
        else:
            first = first_choice(verbose)
            if first is not None:
                return first
    return None


def get_template_path(prefix, template, name):
    return (
        f"djangocms_form_builder/{settings.framework}/{prefix}/{template}/{name}.html"
    )


def get_plugin_template(instance, prefix, name, templates):
    template = getattr(instance, "template", first_choice(templates))
    template_path = get_template_path(prefix, template, name)

    try:
        select_template([template_path])
    except TemplateDoesNotExist:
        # TODO render a warning inside the template
        template_path = get_template_path(prefix, "default", name)

    return template_path


# use mark_safe_lazy to delay the translation when using mark_safe
# otherwise they will not be added to /locale/
# https://docs.djangoproject.com/en/1.11/topics/i18n/translation/#other-uses-of-lazy-in-delayed-translations
mark_safe_lazy = lazy(mark_safe, str)


def add_plugin(placeholder, plugin):
    """CMS version save function to add a plugin to a placeholder"""
    if hasattr(placeholder, "add_plugin"):  # CMS v4?
        placeholder.add_plugin(plugin)
    else:  # CMS < v4
        if plugin.parent:
            plugin.position -= plugin.parent.position + 1
        else:
            plugin.position = 0
        plugin.save()


def delete_plugin(plugin):
    """CMS version save function to delete a plugin (and its descendants) from a placeholder"""
    return plugin.placeholder.delete_plugin(plugin)


def coerce_decimal(value):
    try:
        return decimal.Decimal(value)
    except TypeError:
        return None



# Generated by Django 3.2 on 2023-01-29 19:50

import django.core.serializers.json
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("djangocms_form_builder", "0002_alter_form_cmsplugin_ptr_and_more"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="form",
            name="tag_type",
        ),
        migrations.RemoveField(
            model_name="formfield",
            name="tag_type",
        ),
        migrations.AddField(
            model_name="form",
            name="action_parameters",
            field=models.JSONField(
                blank=True,
                default=dict,
                encoder=django.core.serializers.json.DjangoJSONEncoder,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="form",
            name="captcha_widget",
            field=models.CharField(
                blank=True,
                choices=[("", "-----")],
                default="",
                help_text='Read more in the <a href="https://developers.google.com/recaptcha" target="_blank">documentation</a>.',
                max_length=16,
                verbose_name="captcha widget",
            ),
        ),
    ]



from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from django.utils.encoding import force_str
from django.utils.translation import gettext_lazy as _

from .. import forms
from .. import forms as forms_module
from .. import models, settings
from ..helpers import add_plugin, delete_plugin, insert_fields
from .ajax_plugins import FormPlugin

mixin_factory = settings.get_renderer(forms_module)


class FormElementPlugin(CMSPluginBase):
    top_element = FormPlugin.__name__
    module = _("Forms")
    render_template = f"djangocms_form_builder/{settings.framework}/widgets/base.html"
    change_form_template = "djangocms_frontend/admin/base.html"
    settings_name = _("Settings")
    # Form elements are rendered by the surrounding FormPlugin, hence any change has to be
    # propageted up to the FormPlugin:
    is_local = False

    fieldsets = (
        (
            None,
            {
                "fields": (
                    ("field_label", "field_name"),
                    ("field_required", "field_placeholder"),
                )
            },
        ),
    )

    @classmethod
    def get_parent_classes(cls, slot, page, instance=None):
        """Only valid as indirect child of the cls.top_element"""

        if instance is None:
            return [""]
        parent = instance
        while parent is not None:
            if parent.plugin_type == cls.top_element:
                return super().get_parent_classes(slot, page, instance)
            parent = parent.parent
        return [""]

    def get_fieldsets(self, request, obj=None):
        if hasattr(self, "settings_fields"):
            return insert_fields(
                super().get_fieldsets(request, obj),
                self.settings_fields,
                block=None,
                position=-1,
                blockname=self.settings_name,
                blockattrs=dict(classes=()),
            )
        return super().get_fieldsets(request, obj)

    def render(self, context, instance, placeholder):
        # instance.add_classes("form-control")
        return super().render(context, instance, placeholder)

    def __str__(self):
        return force_str(super().__str__())


@plugin_pool.register_plugin
class CharFieldPlugin(mixin_factory("CharField"), FormElementPlugin):
    name = _("Text")
    model = models.CharField
    form = forms.CharFieldForm
    settings_fields = (("min_length", "max_length"),)


@plugin_pool.register_plugin
class EmailFieldPlugin(mixin_factory("EmailField"), FormElementPlugin):
    name = _("Email")
    model = models.EmailField
    form = forms.EmailFieldForm


@plugin_pool.register_plugin
class URLFieldPlugin(mixin_factory("URLField"), FormElementPlugin):
    name = _("URL")
    model = models.UrlField
    form = forms.UrlFieldForm


@plugin_pool.register_plugin
class DecimalFieldPlugin(mixin_factory("DecimalField"), FormElementPlugin):
    name = _("Decimal")
    model = models.DecimalField
    form = forms.DecimalFieldForm
    settings_fields = ("decimal_places", ("min_value", "max_value"))


@plugin_pool.register_plugin
class IntegerFieldPlugin(mixin_factory("IntegerField"), FormElementPlugin):
    name = _("Integer")
    model = models.IntegerField
    form = forms.IntegerFieldForm
    settings_fields = (("min_value", "max_value"),)


@plugin_pool.register_plugin
class TextareaPlugin(FormElementPlugin):
    name = _("Textarea")
    model = models.TextareaField
    form = forms.TextareaFieldForm
    settings_fields = (
        "field_rows",
        ("min_length", "max_length"),
    )


@plugin_pool.register_plugin
class DateFieldPlugin(FormElementPlugin):
    name = _("Date")
    model = models.DateField
    form = forms.DateFieldForm


@plugin_pool.register_plugin
class DateTimeFieldPlugin(FormElementPlugin):
    name = _("Date and time")
    model = models.DateTimeField
    form = forms.DateTimeFieldForm


@plugin_pool.register_plugin
class TimeFieldPlugin(FormElementPlugin):
    name = _("Time")
    model = models.TimeField
    form = forms.TimeFieldForm


@plugin_pool.register_plugin
class SelectPlugin(mixin_factory("SelectField"), FormElementPlugin):
    name = _("Select")

    model = models.Select
    form = forms.SelectFieldForm
    allow_children = True
    child_classes = ["ChoicePlugin"]

    fieldsets = (
        (
            None,
            {
                "fields": (
                    ("field_label", "field_name"),
                    "field_select",
                    "field_required",
                )
            },
        ),
        (
            _("Choices"),
            {
                "classes": ("collapse",),
                "description": _(
                    "Use this field to quick edit choices. Choices can be added (<kbd>+</kbd>), deleted "
                    "(<kbd>&times;</kbd>) and updated. On the left side enter the value to be stored in the database. "
                    "On the right side enter the text to be shown to the user. The order of choices can be adjusted "
                    "in the structure tree <b>after saving</b> the edits."
                ),
                "fields": ("field_choices",),
            },
        ),
    )

    def save_model(self, request, obj, form, change):
        """Reflects the quick edit changes in the plugin tree"""
        super().save_model(request, obj, form, change)
        child_plugins = obj.get_children()
        children = {}
        for child in child_plugins:
            child_ui = child.djangocms_form_builder_formfield
            children[child_ui.config["value"]] = child_ui
        position = len(child_plugins)
        data = form.cleaned_data
        for value, verbose in data["field_choices"]:
            child = children.pop(value, None)
            if child is not None:  # Need to update?
                if verbose != child.config["verbose"]:
                    child.config["verbose"] = verbose
                    child.save()
            else:  # Not in there, add it!
                add_plugin(
                    obj.placeholder,
                    models.Choice(
                        parent=obj,
                        placeholder=obj.placeholder,
                        position=obj.position + position + 1,
                        language=obj.language,
                        plugin_type=ChoicePlugin.__name__,
                        ui_item=models.Choice.__class__.__name__,
                        config=dict(value=value, verbose=verbose),
                    ),
                )
                position += 1
        for _key, child in children.items():  # Delete remaining
            delete_plugin(child)


@plugin_pool.register_plugin
class ChoicePlugin(mixin_factory("ChoiceField"), FormElementPlugin):
    name = _("Choice")
    module = _("Forms")
    fieldsets = ((None, {"fields": (("value", "verbose"),)}),)
    model = models.Choice
    form = forms.ChoiceForm
    require_parent = True
    parent_classes = ["SelectPlugin"]


@plugin_pool.register_plugin
class BooleanFieldPlugin(mixin_factory("BooleanField"), FormElementPlugin):
    name = _("Boolean")
    model = models.BooleanField
    form = forms.BooleanFieldForm

    fieldsets = (
        (
            None,
            {
                "fields": (
                    ("field_label", "field_name"),
                    "field_as_switch",
                    "field_required",
                )
            },
        ),
    )


@plugin_pool.register_plugin
class SubmitPlugin(mixin_factory("SubmitButton"), FormElementPlugin):
    name = _("Submit button")
    module = _("Forms")

    fieldsets = (
        (
            None,
            {
                "fields": (
                    ("field_name", "field_label"),
                    ("submit_cta",),
                )
            },
        ),
    )
    model = models.SubmitButton
    form = forms.SubmitButtonForm

    render_template = f"djangocms_form_builder/{settings.framework}/widgets/submit.html"



from . import legacy  # noqa F401
from .ajax_plugins import FormPlugin
from .form_plugins import (
    BooleanFieldPlugin,
    CharFieldPlugin,
    ChoicePlugin,
    DateFieldPlugin,
    DateTimeFieldPlugin,
    DecimalFieldPlugin,
    EmailFieldPlugin,
    IntegerFieldPlugin,
    SelectPlugin,
    SubmitPlugin,
    TextareaPlugin,
    TimeFieldPlugin,
    URLFieldPlugin,
)

__all__ = [
    "FormPlugin",
    "BooleanFieldPlugin",
    "CharFieldPlugin",
    "ChoicePlugin",
    "DateFieldPlugin",
    "DateTimeFieldPlugin",
    "DecimalFieldPlugin",
    "EmailFieldPlugin",
    "IntegerFieldPlugin",
    "SelectPlugin",
    "SubmitPlugin",
    "TextareaPlugin",
    "TimeFieldPlugin",
    "URLFieldPlugin",
]



import importlib

from django.conf import settings as django_settings
from django.utils.translation import gettext_lazy as _

EMPTY_CHOICE = (("", "-----"),)

ADMIN_CSS = getattr(
    django_settings,
    "DJANGOCMS_FRONTEND_ADMIN_CSS",
    {},
)


FORM_OPTIONS = getattr(django_settings, "DJANGOCMS_FORMS_OPTIONS", {})
MAIL_TEMPLATE_SETS = getattr(
    django_settings, "DJANGOCMS_MAIL_TEMPLATE_SETS", (("default", _("Default")),)
)

framework = getattr(django_settings, "DJANGOCMS_FRONTEND_FRAMEWORK", "bootstrap5")
theme = getattr(django_settings, "DJANGOCMS_FRONTEND_THEME", "djangocms_frontend")

DEFAULT_SPACER_SIZE_CHOICES = (("mb-3", "Default"),)
TAG_CHOICES = (("div", "div"),)
FORM_TEMPLATE = getattr(
    django_settings,
    "FORM_TEMPLATE",
    f"djangocms_form_builder/{framework}/render/form.html",
)

theme_render_path = f"{theme}.frameworks.{framework}"
theme_forms_path = f"{theme}.forms"

if not getattr(django_settings, "DJANGO_FORM_BUILDER_SPACER_CHOICES", False):
    if not getattr(django_settings, "DJANGOCMS_FRONTEND_SPACER_SIZES", False):
        SPACER_SIZE_CHOICES = DEFAULT_SPACER_SIZE_CHOICES
    else:
        SPACER_SIZE_CHOICES = [
            (f"mb-{key}", value)
            for key, value in django_settings.DJANGOCMS_FRONTEND_SPACER_SIZES
        ]
else:
    SPACER_SIZE_CHOICES = django_settings.DJANGO_FORM_BUILDER_SPACER_CHOICES


def render_factory(cls, theme_module, render_module):
    parents = tuple(
        getattr(module, cls, None)
        for module in (theme_module, render_module)
        if module is not None and getattr(module, cls, None) is not None
    )
    return type(cls, parents, dict())  # Empty Mix


def get_mixins(naming, theme_path, mixin_path):
    try:
        theme_module = importlib.import_module(theme_path) if theme_path else None
    except ModuleNotFoundError:
        theme_module = None
    try:
        render_module = importlib.import_module(mixin_path) if mixin_path else None
    except ModuleNotFoundError:
        render_module = None

    return lambda name: render_factory(
        naming.format(name=name), theme_module, render_module
    )


def get_renderer(my_module):
    if not isinstance(my_module, str):
        my_module = my_module.__name__
    return get_mixins(
        "{name}RenderMixin", theme_render_path, f"{my_module}.frameworks.{framework}"
    )


def get_forms(my_module):
    if not isinstance(my_module, str):
        my_module = my_module.__name__
    return get_mixins(
        "{name}FormMixin", theme_forms_path, f"{my_module}.frameworks.{framework}"
    )



import hashlib

from cms.models import CMSPlugin
from django.core.exceptions import ValidationError
from django.http import Http404, JsonResponse, QueryDict
from django.shortcuts import get_object_or_404
from django.utils.crypto import get_random_string
from django.utils.translation import gettext as _
from django.views import View

_formview_pool = {}


def register_form_view(cls, slug=None):
    """
    Registers a Widget (with type defined by cls) and slug
    :type cls: class
    :type slug: string to instantiate dashboard_widget
    """
    if not slug:
        slug = get_random_string(length=12)
    key = hashlib.sha384(slug.encode("utf-8")).hexdigest()
    if key in _formview_pool:
        assert _formview_pool[key][0] == cls, _(
            "Only unique slugs accepted for form views"
        )
    _formview_pool[key] = (cls, slug, key)
    return key


class AjaxView(View):
    # @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        # is_ajax = request.headers.get('X-Requested-With') == "XMLHttpRequest"
        if request.accepts("application/json"):
            if request.method == "GET" and "get" in self.http_method_names:
                return self.ajax_get(request, *args, **kwargs)
            elif request.method == "POST" and "post" in self.http_method_names:
                return self.ajax_post(request, *args, **kwargs)
        return super().dispatch(request, *args, **kwargs)

    @staticmethod
    def decode_path(path):
        params = {}
        for element in path.split(","):
            if "=" in element:
                params[element.split("=", 1)[0]] = element.split("=", 1)[1]
            elif "%3D" in element:
                params[element.split("%3D", 1)[0]] = element.split("%3D", 1)[1]
            else:
                params[element] = True
        return params

    @staticmethod
    def plugin_instance(pk):
        plugin = get_object_or_404(CMSPlugin, pk=pk)
        plugin.__class__ = plugin.get_plugin_class()
        instance = (
            plugin.model.objects.get(cmsplugin_ptr=plugin.id)
            if hasattr(plugin.model, "cmsplugin_ptr")
            else plugin
        )
        return plugin, instance

    def ajax_post(self, request, *args, **kwargs):
        if "instance_id" in kwargs:
            plugin, instance = self.plugin_instance(kwargs["instance_id"])
            if hasattr(plugin, "ajax_post"):
                request.POST = QueryDict(request.body)
                try:
                    params = (
                        self.decode_path(kwargs["parameter"])
                        if "parameter" in kwargs
                        else {}
                    )
                    return plugin.ajax_post(request, instance, params)
                except ValidationError as error:
                    return JsonResponse({"result": "error", "msg": str(error.args[0])})
            else:
                raise Http404()
        elif "form_id" in kwargs:
            if kwargs["form_id"] in _formview_pool:
                form_id = kwargs.pop("form_id")
                instance = _formview_pool[form_id][0](*args, **kwargs)
                if hasattr(instance, "ajax_post"):
                    return instance.ajax_post(request, *args, **kwargs)
                elif hasattr(instance, "post"):
                    return instance.post(request, *args, **kwargs)
            raise Http404()
        raise Http404()

    def ajax_get(self, request, *args, **kwargs):
        if "instance_id" in kwargs:
            plugin, instance = self.plugin_instance(kwargs["instance_id"])
            if hasattr(plugin, "ajax_get"):
                request.GET = QueryDict(request.body)
                try:
                    params = (
                        self.decode_path(kwargs["parameter"])
                        if "parameter" in kwargs
                        else {}
                    )
                    return plugin.ajax_get(request, instance, params)
                except ValidationError as error:
                    return JsonResponse({"result": "error", "msg": str(error.args[0])})
        elif "form_id" in kwargs:
            if kwargs["form_id"] in _formview_pool:
                form_id = kwargs.pop("form_id")
                instance = _formview_pool[form_id][0](**kwargs)
                if hasattr(instance, "ajax_get"):
                    return instance.ajax_get(request, *args, **kwargs)
                elif hasattr(instance, "get"):
                    return instance.get(request, *args, **kwargs)
            raise Http404()
        raise Http404()



from django.apps import apps
from django.conf import settings
from django.utils.translation import gettext_lazy as _

from .helpers import coerce_decimal

CAPTCHA_WIDGETS = {}
CAPTCHA_FIELDS = {}
CAPTCHA_CHOICES = ()

if apps.is_installed("captcha"):
    """Soft dependency on django-captcha for reCaptchaField"""

    from captcha.fields import ReCaptchaField  # NOQA
    from captcha.widgets import ReCaptchaV2Checkbox, ReCaptchaV2Invisible  # NOQA

    CAPTCHA_WIDGETS["v2-checkbox"] = ReCaptchaV2Checkbox
    CAPTCHA_WIDGETS["v2-invisible"] = ReCaptchaV2Invisible

    CAPTCHA_FIELDS["v2-checkbox"] = ReCaptchaField
    CAPTCHA_FIELDS["v2-invisible"] = ReCaptchaField

    CAPTCHA_CHOICES += (
        ("v2-checkbox", f"reCaptcha - {_('v2 checkbox')}"),
        ("v2-invisible", f"reCaptcha - {_('v2 invisible')}"),
    )

if apps.is_installed("hcaptcha"):
    """Soft dependency on django-hcaptcha for hcaptcha"""

    from hcaptcha.fields import hCaptchaField  # NOQA
    from hcaptcha.widgets import hCaptchaWidget  # NOQA

    CAPTCHA_FIELDS["hcaptcha"] = hCaptchaField
    CAPTCHA_WIDGETS["hcaptcha"] = hCaptchaWidget

    CAPTCHA_CHOICES += (("hcaptcha", _("hCaptcha")),)

if len(CAPTCHA_CHOICES) > 0:
    installed = True
else:
    installed = False


def get_recaptcha_field(instance):
    config = instance.captcha_config
    widget_params = {
        "attrs": {
            key: value
            for key, value in config.get("captcha_config", {}).items()
            if key.startswith("data-")
        },
        "api_params": {
            key: value
            for key, value in config.get("captcha_config", {}).items()
            if not key.startswith("data-")
        },
    }
    widget_params["attrs"]["no_field_sep"] = True
    if config.get("captcha_widget", "") == "v3":
        widget_params["attrs"]["required_score"] = coerce_decimal(
            config.get("captcha_requirement", 0.5)
        )  # installing recaptcha 3 ?
    if not widget_params["api_params"]:
        del widget_params["api_params"]
    return CAPTCHA_FIELDS[instance.captcha_widget](
        widget=CAPTCHA_WIDGETS[instance.captcha_widget](**widget_params), label=""
    )


keys_available = installed and (
    hasattr(settings, "RECAPTCHA_PUBLIC_KEY")
    and hasattr(settings, "RECAPTCHA_PRIVATE_KEY")
)

field_name = "captcha_field"
RECAPTCHA_PUBLIC_KEY = getattr(settings, "RECAPTCHA_PUBLIC_KEY", "")



import importlib

from django.utils.translation import gettext_lazy as _

from . import settings

framework = importlib.import_module(
    f"djangocms_form_builder.frontends.{settings.framework}",  # TODO
)

default_attr = framework.default_attr  # NOQA
attr_dict = framework.attr_dict  # NOQA
DEFAULT_FIELD_SEP = framework.DEFAULT_FIELD_SEP  # NOQA

# default_attr = settings.default_attr  # NOQA
# attr_dict = settings.attr_dict  # NOQA
# DEFAULT_FIELD_SEP = settings.DEFAULT_FIELD_SEP  # NOQA

CHOICE_FIELDS = (
    (
        _("Single choice"),
        (
            ("select", _("Drop down")),
            ("radio", _("Radio buttons")),
        ),
    ),
    (
        _("Multiple choice"),
        (
            ("checkbox", _("Checkboxes")),
            ("multiselect", _("List")),
        ),
    ),
)



# Generated by Django 3.2.13 on 2022-06-01 10:21

import re

import django.core.serializers.json
import django.core.validators
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import djangocms_form_builder.fields


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("cms", "0022_auto_20180620_1551"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Form",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        related_name="djangocms_form_builder_form",
                        serialize=False,
                        to="cms.cmsplugin",
                    ),
                ),
                (
                    "form_selection",
                    models.CharField(
                        blank=True, default="", max_length=256, verbose_name="Form"
                    ),
                ),
                (
                    "form_name",
                    models.CharField(
                        blank=True,
                        default="",
                        max_length=256,
                        validators=[
                            django.core.validators.RegexValidator(
                                re.compile("^[-a-zA-Z0-9_]+\\Z"),
                                "Enter a valid “slug” consisting of letters, numbers, underscores or hyphens.",
                                "invalid",
                            )
                        ],
                        verbose_name="Form name",
                    ),
                ),
                (
                    "form_login_required",
                    models.BooleanField(
                        blank=True,
                        default=False,
                        help_text="To avoid issues with user experience use this type of form only on pages, which require login.",
                        verbose_name="Login required to submit form",
                    ),
                ),
                (
                    "form_unique",
                    models.BooleanField(
                        default=False,
                        help_text='Requires "Login required" to be checked to work.',
                        verbose_name="User can reopen form",
                    ),
                ),
                (
                    "form_floating_labels",
                    models.BooleanField(default=False, verbose_name="Floating labels"),
                ),
                (
                    "form_spacing",
                    models.CharField(
                        max_length=16, verbose_name="Margin between fields"
                    ),
                ),
                (
                    "form_actions",
                    models.CharField(
                        blank=True,
                        max_length=1024,
                        verbose_name="Actions to be taken after form submission",
                    ),
                ),
                (
                    "attributes",
                    djangocms_form_builder.fields.AttributesField(
                        blank=True, default=dict, verbose_name="Attributes"
                    ),
                ),
                (
                    "captcha_widget",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("", "-----"),
                            ("v2-checkbox", "v2 checkbox"),
                            ("v2-invisible", "v2 invisible"),
                        ],
                        default="",
                        help_text='Read more in the <a href="https://developers.google.com/recaptcha" target="_blank">documentation</a>.',
                        max_length=16,
                        verbose_name="reCaptcha widget",
                    ),
                ),
                (
                    "captcha_requirement",
                    models.DecimalField(
                        decimal_places=2,
                        default=0.5,
                        help_text="Only for reCaptcha v3: Minimum score required to accept challenge.",
                        max_digits=3,
                        null=True,
                        verbose_name="Minimum score requirement",
                    ),
                ),
                (
                    "captcha_config",
                    djangocms_form_builder.fields.AttributesField(
                        blank=True,
                        default=dict,
                        help_text='The reCAPTCHA widget supports several <a href="https://developers.google.com/recaptcha/docs/display#render_param" target="_blank">data attributes</a> that customize the behaviour of the widget, such as <code>data-theme</code>, <code>data-size</code>. The reCAPTCHA api supports several <a href="https://developers.google.com/recaptcha/docs/display#javascript_resource_apijs_parameters" target="_blank">parameters</a>. Add these api parameters as attributes, e.g. <code>hl</code> to set the language.',
                        verbose_name="Recaptcha configuration parameters",
                    ),
                ),
                (
                    "tag_type",
                    djangocms_form_builder.fields.TagTypeField(
                        choices=[("div", "div")],
                        default="div",
                        help_text="Select the HTML tag to be used.",
                        max_length=255,
                        verbose_name="Tag type",
                    ),
                ),
            ],
            options={
                "verbose_name": "Form",
            },
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="FormField",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        related_name="djangocms_form_builder_formfield",
                        serialize=False,
                        to="cms.cmsplugin",
                    ),
                ),
                ("ui_item", models.CharField(max_length=30)),
                (
                    "tag_type",
                    djangocms_form_builder.fields.TagTypeField(
                        blank=True,
                        choices=[
                            ("div", "div"),
                        ],
                        default="div",
                        help_text="Select the HTML tag to be used.",
                        max_length=255,
                        verbose_name="Tag type",
                    ),
                ),
                (
                    "config",
                    models.JSONField(
                        default=dict,
                        encoder=django.core.serializers.json.DjangoJSONEncoder,
                    ),
                ),
            ],
            options={
                "verbose_name": "Form field item",
            },
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="FormEntry",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("form_name", models.SlugField(verbose_name="Form")),
                (
                    "entry_data",
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=django.core.serializers.json.DjangoJSONEncoder,
                    ),
                ),
                ("html_headers", models.JSONField(blank=True, default=dict)),
                ("entry_created_at", models.DateTimeField(auto_now_add=True)),
                ("entry_updated_at", models.DateTimeField(auto_now=True)),
                (
                    "form_user",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="User",
                    ),
                ),
            ],
            options={
                "verbose_name": "Form entry",
                "verbose_name_plural": "Form entries",
            },
        ),
        migrations.CreateModel(
            name="BooleanField",
            fields=[],
            options={
                "verbose_name": "Boolean field",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("djangocms_form_builder.formfield",),
        ),
        migrations.CreateModel(
            name="CharField",
            fields=[],
            options={
                "verbose_name": "Character field",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("djangocms_form_builder.formfield",),
        ),
        migrations.CreateModel(
            name="Choice",
            fields=[],
            options={
                "verbose_name": "Choice",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("djangocms_form_builder.formfield",),
        ),
        migrations.CreateModel(
            name="DateField",
            fields=[],
            options={
                "verbose_name": "Date field",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("djangocms_form_builder.formfield",),
        ),
        migrations.CreateModel(
            name="DateTimeField",
            fields=[],
            options={
                "verbose_name": "Date field",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("djangocms_form_builder.formfield",),
        ),
        migrations.CreateModel(
            name="DecimalField",
            fields=[],
            options={
                "verbose_name": "Decimal field",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("djangocms_form_builder.formfield",),
        ),
        migrations.CreateModel(
            name="EmailField",
            fields=[],
            options={
                "verbose_name": "Email field",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("djangocms_form_builder.formfield",),
        ),
        migrations.CreateModel(
            name="IntegerField",
            fields=[],
            options={
                "verbose_name": "Integer field",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("djangocms_form_builder.formfield",),
        ),
        migrations.CreateModel(
            name="Select",
            fields=[],
            options={
                "verbose_name": "Select",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("djangocms_form_builder.formfield",),
        ),
        migrations.CreateModel(
            name="SubmitButton",
            fields=[],
            options={
                "verbose_name": "Submit button",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("djangocms_form_builder.formfield",),
        ),
        migrations.CreateModel(
            name="TextareaField",
            fields=[],
            options={
                "verbose_name": "Text field",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("djangocms_form_builder.formfield",),
        ),
        migrations.CreateModel(
            name="TimeField",
            fields=[],
            options={
                "verbose_name": "Date field",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("djangocms_form_builder.formfield",),
        ),
        migrations.CreateModel(
            name="UrlField",
            fields=[],
            options={
                "verbose_name": "URL field",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("djangocms_form_builder.formfield",),
        ),
    ]



from importlib import import_module

from django.apps import AppConfig
from django.conf import settings
from django.urls import NoReverseMatch, clear_url_caches, include, path, reverse
from django.utils.translation import gettext_lazy as _


class FormsConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "djangocms_form_builder"
    verbose_name = _("Form builder")

    def ready(self):
        """Install the URLs"""
        try:
            reverse("form_builder:ajaxview", args=(1,))
        except NoReverseMatch:  # Not installed yet
            urlconf_module = import_module(settings.ROOT_URLCONF)
            urlconf_module.urlpatterns = [
                path(
                    "@form-builder/",
                    include(
                        "djangocms_form_builder.urls",
                        namespace="form_builder",
                    ),
                )
            ] + urlconf_module.urlpatterns
            clear_url_caches()
            reverse("form_builder:ajaxview", args=(1,))



# Generated by Django 4.0 on 2022-10-19 10:08

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("cms", "0022_auto_20180620_1551"),
        ("djangocms_form_builder", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="form",
            name="cmsplugin_ptr",
            field=models.OneToOneField(
                auto_created=True,
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                related_name="%(app_label)s_%(class)s",
                serialize=False,
                to="cms.cmsplugin",
            ),
        ),
        migrations.AlterField(
            model_name="formfield",
            name="cmsplugin_ptr",
            field=models.OneToOneField(
                auto_created=True,
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                related_name="%(app_label)s_%(class)s",
                serialize=False,
                to="cms.cmsplugin",
            ),
        ),
    ]



default_attr = dict(
    input="form-control",
    label="form-label",
    div="",
)

attr_dict = dict(
    Select=dict(input="form-select"),
    SelectMultiple=dict(input="form-select"),
    NullBooleanSelect=dict(input="form-select"),
    RadioSelect=dict(
        input="form-check-input", label="form-check-label", div="form-check"
    ),
    CheckboxInput=dict(
        input="form-check-input", label="form-check-label", div="form-check"
    ),
    ButtonRadio=dict(input="btn-check", label="btn btn-outline-primary"),
    ButtonCheckbox=dict(input="btn-check", label="btn btn-outline-primary"),
)

DEFAULT_FIELD_SEP = "mb-3"
